==> clean_android_design-0.0.6.tar.gz: lib/src/fields/selectable_list_field_widget.dart <==
import 'package:clean_android_design/clean_android_design.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class SelectableListFieldWidget<TMain, TItem extends ReadOnlyDatabaseItem>
    extends StatefulWidget {
  final bool isFirst;
  final bool isLast;

  final void Function(TMain item, List<DocumentReference> selectedItemRefs)
      saveToItem;
  final List<DocumentReference> Function(TMain item) getFromItem;

  final bool Function(TMain, TItem) allItemsLambda;
  final ISelectableListItem Function(
      BuildContext context, TMain mainItem, TItem item) itemBuilder;
  final Comparable Function(TItem item) sortProperty;

  final bool bound;
  final bool isEditable;

  final Widget noItemWidget;
  final Widget noItemWidgetEditable;

  final Widget topWidget;

  const SelectableListFieldWidget({
    Key key,
    this.saveToItem,
    this.getFromItem,
    this.allItemsLambda,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemWidgetEditable,
    this.sortProperty,
    this.topWidget,
  })  : isFirst = false,
        isLast = false,
        bound = true,
        isEditable = null,
        assert(getFromItem != null),
        assert(saveToItem != null),
        super(key: key);

  const SelectableListFieldWidget.unbound({
    Key key,
    this.saveToItem,
    this.getFromItem,
    this.allItemsLambda,
    this.isEditable,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemWidgetEditable,
    this.sortProperty,
    this.topWidget,
  })  : isFirst = false,
        isLast = false,
        bound = false,
        assert(getFromItem != null),
        assert(saveToItem != null),
        super(key: key);

  @override
  _SelectableListFieldWidgetState<TMain, TItem> createState() =>
      _SelectableListFieldWidgetState<TMain, TItem>();
}

class _SelectableListFieldWidgetState<TMain, TItem extends ReadOnlyDatabaseItem>
    extends State<SelectableListFieldWidget<TMain, TItem>> {
  TMain mainItem;
  List<TItem> allItems;

  @override
  Widget build(BuildContext context) {
    if (widget.bound) mainItem = Provider.of<TMain>(context);

    allItems = Provider.of<List<TItem>>(context)
        .where((item) => widget.allItemsLambda?.call(mainItem, item) ?? true)
        .toList();

    if (widget.sortProperty != null)
      allItems.sort((i1, i2) =>
          widget.sortProperty(i1).compareTo(widget.sortProperty(i2)));

    if (widget.isEditable == null)
      return Consumer<bool>(
          builder: (context, isEditable, __) =>
              _getDefautFieldWidget(isEditable));
    else
      return _getDefautFieldWidget(widget.isEditable);
  }

  Widget _getDefautFieldWidget(bool isEditable) {
    var items = isEditable
        ? allItems
        : allItems
            .where((e) => widget
                .getFromItem(mainItem)
                .any((ref) => ref.documentID == e.id))
            .toList();

    if (items.length == 0) if (isEditable)
      return widget.noItemWidgetEditable ?? widget.noItemWidget ?? Container();
    else
      return widget.noItemWidget ?? Container();

    return Column(
      children: <Widget>[
        widget.topWidget ?? Container(),
        ItemSectionWidget.builder(
          itemBuilder: (context, index) => widget
              .itemBuilder(context, mainItem, items[index])
              .copyWithSelection(
                isSelected: isEditable
                    ? widget
                        .getFromItem(mainItem)
                        .any((ref) => ref.documentID == items[index].id)
                    : false,
                onTapList: isEditable
                    ? () {
                        _changeSelectionFor(items[index]);
                        return false;
                      }
                    : null,
              ),
          itemCount: items.length,
        ),
      ],
    );
  }

  _changeSelectionFor(TItem item) {
    var selectedItems = widget.getFromItem(mainItem);
    if (selectedItems.any((ref) => ref.documentID == item.id))
      selectedItems.removeWhere((ref) => ref.documentID == item.id);
    else
      selectedItems
          .add(Firestore.instance.document("${TItem.toString()}/${item.id}"));

    setState(() => widget.saveToItem(mainItem, selectedItems));
  }
}

==> clean_android_design-0.0.14.tar.gz: lib/src/fields/selectable_list_field_widget.dart <==
import 'package:clean_android_design/clean_android_design.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class SelectableListFieldWidget<TMain, TItem extends ReadOnlyDatabaseItem>
    extends StatefulWidget {
  final bool isFirst;
  final bool isLast;

  final void Function(TMain item, List<DocumentReference> selectedItemRefs)
      saveToItem;
  final List<DocumentReference> Function(TMain item) getFromItem;

  final bool Function(TMain, TItem) allItemsLambda;
  final ISelectableListItem Function(
      BuildContext context, TMain mainItem, TItem item) itemBuilder;
  final Comparable Function(TItem item) sortProperty;

  final bool bound;
  final bool isEditable;

  final Widget noItemWidget;
  final Widget noItemWidgetEditable;

  final Widget topWidget;

  const SelectableListFieldWidget({
    Key key,
    this.saveToItem,
    this.getFromItem,
    this.allItemsLambda,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemWidgetEditable,
    this.sortProperty,
    this.topWidget,
  })  : isFirst = false,
        isLast = false,
        bound = true,
        isEditable = null,
        assert(getFromItem != null),
        assert(saveToItem != null),
        super(key: key);

  const SelectableListFieldWidget.unbound({
    Key key,
    this.saveToItem,
    this.getFromItem,
    this.allItemsLambda,
    this.isEditable,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemWidgetEditable,
    this.sortProperty,
    this.topWidget,
  })  : isFirst = false,
        isLast = false,
        bound = false,
        assert(getFromItem != null),
        assert(saveToItem != null),
        super(key: key);

  @override
  _SelectableListFieldWidgetState<TMain, TItem> createState() =>
      _SelectableListFieldWidgetState<TMain, TItem>();
}

class _SelectableListFieldWidgetState<TMain, TItem extends ReadOnlyDatabaseItem>
    extends State<SelectableListFieldWidget<TMain, TItem>> {
  TMain mainItem;
  List<TItem> allItems;

  @override
  Widget build(BuildContext context) {
    if (widget.bound) mainItem = Provider.of<TMain>(context);

    allItems = Provider.of<List<TItem>>(context)
        .where((item) => widget.allItemsLambda?.call(mainItem, item) ?? true)
        .toList();

    if (widget.sortProperty != null)
      allItems.sort((i1, i2) =>
          widget.sortProperty(i1).compareTo(widget.sortProperty(i2)));

    if (widget.isEditable == null)
      return Consumer<bool>(
          builder: (context, isEditable, __) =>
              _getDefautFieldWidget(isEditable));
    else
      return _getDefautFieldWidget(widget.isEditable);
  }

  Widget _getDefautFieldWidget(bool isEditable) {
    var items = isEditable
        ? allItems
        : allItems
            .where((e) =>
                widget.getFromItem(mainItem).any((ref) => ref.id == e.id))
            .toList();

    if (items.length == 0) if (isEditable)
      return widget.noItemWidgetEditable ?? widget.noItemWidget ?? Container();
    else
      return widget.noItemWidget ?? Container();

    return Column(
      children: <Widget>[
        widget.topWidget ?? Container(),
        ItemSectionWidget.builder(
          itemBuilder: (context, index) => widget
              .itemBuilder(context, mainItem, items[index])
              .copyWithSelection(
                isSelected: isEditable
                    ? widget
                        .getFromItem(mainItem)
                        .any((ref) => ref.id == items[index].id)
                    : false,
                onTapList: isEditable
                    ? () {
                        _changeSelectionFor(items[index]);
                        return false;
                      }
                    : null,
              ),
          itemCount: items.length,
        ),
      ],
    );
  }

  _changeSelectionFor(TItem item) {
    var selectedItems = widget.getFromItem(mainItem);
    if (selectedItems.any((ref) => ref.id == item.id))
      selectedItems.removeWhere((ref) => ref.id == item.id);
    else
      selectedItems.add(
          FirebaseFirestore.instance.doc("${TItem.toString()}/${item.id}"));

    setState(() => widget.saveToItem(mainItem, selectedItems));
  }
}
